#!/usr/bin/env python3
# Copyright (C) 2023 - Teclib'ERP (<https://www.teclib-erp.com>).
# License LGPL-3.0 or later (https://www.gnu.org/licenses/lgpl.html).

from src.odoo_configurator.__main__ import main

if __name__ == '__main__':
    main()
